#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = __importDefault(require("yargs/yargs"));
const helpers_1 = require("yargs/helpers");
const _1 = require(".");
(0, yargs_1.default)((0, helpers_1.hideBin)(process.argv))
    .scriptName('parikshan')
    .command('$0  <files..>', 'Compiles code to performance.measure the functions', yargs => {
    return yargs.positional('files', {
        description: 'One or more files or glob patterns to compile',
    });
}, argv => {
    const compiler = new _1.Compiler();
    compiler.compile(argv.files, argv.outputDir, argv.sourceLoc);
})
    .example([
    [
        '$0 "{,!(node_modules)/**/}*.js" -o output',
        'Compile all js files except for files under node_modules to output folder',
    ],
    [
        '$0 build/src/**/*.js -so output',
        'Compile to output folder with source location captured',
    ],
])
    .option('output-dir', {
    alias: 'o',
    type: 'string',
    description: 'Output directory',
    default: 'build/parikshan',
})
    .option('source-loc', {
    alias: 's',
    type: 'boolean',
    description: 'Capture line, column number in performance measurement',
    default: false,
})
    .updateStrings({
    'Positionals:': 'Positional Arguments:',
})
    .alias('help', 'h')
    .alias('version', 'v')
    .parse();
//# sourceMappingURL=cli.js.map