import { VariableDeclaration } from 'jscodeshift';
/**
 * Compiler class
 * @example
 * How to use it:
 * ```javascript
 * const compiler = new Compiler()
 * const compiledFileContents = await compiler.compile(["build/src/**\/*.js"], "output", true)
 * ```
 */
export declare class Compiler {
    #private;
    /** AST for requiring parikshan to be added at start of each file */
    static parikshanAST: VariableDeclaration;
    /**
     * Wraps source code's functions and their arguments
     * which could possibly have function value at runtime into {@link parikshan | `parikshan`} function
     * @param src - original source code
     * @param filePath - source code file path
     * @param captureSourceLocation - whether to pass source code coordinates as argument to {@link parikshan | `parikshan`} function
     * @returns transformed source code
     */
    transform(src: string, filePath?: string, captureSourceLocation?: boolean): string;
    /**
     * Reads source from matched files and writes transformed
     * source to output dir keeping the file structure
     * @param globPaths - input file glob patterns
     * @param destPath - output dir
     * @param captureSourceLocation - set it to true if wanted to capture line, column number and filename in performance entry
     */
    compile(globPaths: string[], destPath: string, captureSourceLocation?: boolean): Promise<void[]>;
}
