"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Compiler_instances, _Compiler_wrapper;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Compiler = void 0;
const path_1 = __importDefault(require("path"));
const jscodeshift_1 = __importDefault(require("jscodeshift"));
const recast = __importStar(require("recast"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const fs_extra_1 = __importDefault(require("fs-extra"));
/**
 * Compiler class
 * @example
 * How to use it:
 * ```javascript
 * const compiler = new Compiler()
 * const compiledFileContents = await compiler.compile(["build/src/**\/*.js"], "output", true)
 * ```
 */
class Compiler {
    constructor() {
        _Compiler_instances.add(this);
    }
    /**
     * Wraps source code's functions and their arguments
     * which could possibly have function value at runtime into {@link parikshan | `parikshan`} function
     * @param src - original source code
     * @param filePath - source code file path
     * @param captureSourceLocation - whether to pass source code coordinates as argument to {@link parikshan | `parikshan`} function
     * @returns transformed source code
     */
    transform(src, filePath, captureSourceLocation) {
        const validCalleeTypes = [
            'ArrowFunctionExpression',
            'FunctionExpression',
            'Identifier',
            'MemberExpression',
            'SequenceExpression',
        ];
        return (0, jscodeshift_1.default)(src)
            .find(jscodeshift_1.default.CallExpression)
            .filter(c => {
            const callee = c.value.callee;
            const isSuperMemberExpr = callee.type === 'MemberExpression' && callee.object.type === 'Super';
            const isSuper = callee.type === 'Super';
            if (isSuperMemberExpr || isSuper) {
                return false;
            }
            return validCalleeTypes.includes(callee.type);
        })
            .replaceWith(c => {
            const srcLoc = c.value.loc;
            let location;
            if (captureSourceLocation) {
                location = recast.parse(`(${JSON.stringify({
                    start: {
                        line: srcLoc === null || srcLoc === void 0 ? void 0 : srcLoc.start.line,
                        column: srcLoc === null || srcLoc === void 0 ? void 0 : srcLoc.start.column,
                    },
                    end: {
                        line: srcLoc === null || srcLoc === void 0 ? void 0 : srcLoc.end.line,
                        column: srcLoc === null || srcLoc === void 0 ? void 0 : srcLoc.end.column,
                    },
                    filename: filePath || '',
                })})`).program.body[0].expression;
            }
            c.value.callee = __classPrivateFieldGet(this, _Compiler_instances, "m", _Compiler_wrapper).call(this, c.value.callee, true, location);
            c.value.arguments = c.value.arguments.map(argument => {
                if (validCalleeTypes.includes(argument.type)) {
                    return __classPrivateFieldGet(this, _Compiler_instances, "m", _Compiler_wrapper).call(this, argument, false, location);
                }
                else {
                    return argument;
                }
            });
            return c.value;
        })
            .closest(jscodeshift_1.default.Program)
            .replaceWith(p => {
            p.value.body = [Compiler.parikshanAST, ...p.value.body];
            return p.value;
        })
            .toSource();
    }
    /**
     * Reads source from matched files and writes transformed
     * source to output dir keeping the file structure
     * @param globPaths - input file glob patterns
     * @param destPath - output dir
     * @param captureSourceLocation - set it to true if wanted to capture line, column number and filename in performance entry
     */
    async compile(globPaths, destPath, captureSourceLocation = false) {
        await fs_extra_1.default.remove(destPath);
        const files = await (0, fast_glob_1.default)(globPaths, { dot: true });
        const compiledFiles = files.map(async (filePath) => {
            const src = await fs_extra_1.default.readFile(filePath, 'utf8');
            const output = this.transform(src, filePath, captureSourceLocation);
            const outputPath = path_1.default.join(destPath, path_1.default
                .relative(destPath, filePath)
                .split('/')
                .filter(seg => seg !== '..')
                .join('/'));
            await fs_extra_1.default.ensureFile(outputPath);
            return fs_extra_1.default.writeFile(path_1.default.join(outputPath), output);
        });
        return Promise.all(compiledFiles);
    }
}
exports.Compiler = Compiler;
_Compiler_instances = new WeakSet(), _Compiler_wrapper = function _Compiler_wrapper(callee, isCallee = false, location) {
    if (callee.type === 'MemberExpression' &&
        callee.object.type !== 'CallExpression' &&
        isCallee) {
        callee = jscodeshift_1.default.conditionalExpression(jscodeshift_1.default.callExpression(jscodeshift_1.default.identifier('parikshan.usesThisOrNative'), [
            callee,
        ]), jscodeshift_1.default.callExpression(jscodeshift_1.default.memberExpression(callee, jscodeshift_1.default.identifier('bind')), [callee.object]), callee);
    }
    else if (callee.type === 'SequenceExpression') {
        // NOTE :- ignoring below line messes up parenthesis marking in output code, the expressions of SequenceExpression becomes individual arguments
        callee.extra = {
            parenthesized: true,
        };
    }
    const parikshanArgs = [callee];
    if (location)
        parikshanArgs.push(location);
    return jscodeshift_1.default.callExpression(jscodeshift_1.default.identifier('parikshan'), parikshanArgs);
};
/** AST for requiring parikshan to be added at start of each file */
Compiler.parikshanAST = recast.parse('\nconst {parikshan} = require("parikshan/build/src/parikshan");\n').program.body[0];
//# sourceMappingURL=compiler.js.map