"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initMongoPerfSubscriber = void 0;
const mongodb_1 = require("mongodb");
const perf_hooks_1 = require("perf_hooks");
const collections = {};
/**
 * Connects to db and and saves performance measurements in time series collection,
 * if collection doesn't exist then a new time series collection with given name will be created
 * @param dbDetails - mongo db and collection details
 */
const initMongoPerfSubscriber = async (dbDetails) => {
    const client = new mongodb_1.MongoClient(dbDetails.dbConnectionString);
    await client.connect();
    const db = client.db(dbDetails.dbName);
    const currentCollections = await db.collections({
        nameOnly: true,
    });
    const serverInfo = await db.admin().serverInfo();
    if (serverInfo.version >= '5' &&
        !currentCollections.find(collection => collection.collectionName === dbDetails.collectionName)) {
        const opts = {
            timeField: 'calledAt',
            metaField: 'functionName',
            granularity: 'seconds',
        };
        collections.parikshans = await db.createCollection(dbDetails.collectionName, {
            timeseries: opts,
            expireAfterSeconds: dbDetails.deleteAfterSeconds,
        });
    }
    else {
        collections.parikshans = db.collection(dbDetails.collectionName);
    }
};
exports.initMongoPerfSubscriber = initMongoPerfSubscriber;
// not keeping it in mongoentry since multiple function calls will create multiple observers
const observer = new perf_hooks_1.PerformanceObserver(entryList => {
    if (collections.parikshans) {
        const entries = entryList.getEntries();
        const parikshans = entries
            .filter(entry => {
            var _a;
            return entry.name === 'parikshan' &&
                ((_a = entry.detail) === null || _a === void 0 ? void 0 : _a.functionName) !== 'initMongoPerfSubscriber';
        })
            .map((entry) => {
            const entryJSON = entry.toJSON();
            entryJSON.calledAt = entryJSON.detail.calledAt;
            entryJSON.functionName = entryJSON.detail.functionName;
            return entryJSON;
        });
        if (parikshans.length)
            collections.parikshans.insertMany(parikshans);
    }
});
observer.observe({
    entryTypes: ['measure'],
});
//# sourceMappingURL=mongoPerfSubscriber.js.map