import { SourceLocation } from 'jscodeshift';
export interface AnyFunction {
    (...args: any[]): any;
    [key: symbol]: (...args: any[]) => any;
}
/**
 * Wraps input function with performance measurement.
 * If the wrapped function returns a promise,
 * a finally handler will be attached to the promise and the duration will be reported once the finally handler is invoked
 * @param fn - input function
 * @param loc - location of function in original source code
 * @returns function wrapped with performance hooks
 */
export declare const parikshan: {
    (fn: AnyFunction, loc?: SourceLocation | undefined): (...args: any[]) => any;
    /**
     * Checks function string for native code/this.
     * @param fn - any function
     * @returns true if function string contains native code/this variable
     */
    usesThisOrNative(fn: AnyFunction): boolean;
};
export interface ParikshanPerformaceEntry {
    /** PerformanceEntry name */
    name: 'parikshan';
    /** A high res timeStamp representing the time value of the duration of the function */
    duration: number;
    /** A high res timeStamp representing the starting time for the performance metric. Not a UNIX timestamp */
    startTime: number;
    /** Entry type */
    entryType: 'measure';
    /** Extra details */
    detail: {
        /** Function name. 'anonymous' if function doesn't have any name */
        functionName: string;
        /** list of function's stringified arguments */
        arguments: string[];
        /** `Date` object for function called at */
        calledAt: Date;
        /** `Date` object for function returned at */
        returnedAt: Date;
        /** Location of function in source code. Will be only present when compiled with -s flag else undefined */
        location: {
            start: {
                line: number;
                column: number;
            };
            end: {
                line: number;
                column: number;
            };
            filename: string;
        } | undefined;
    };
}
